/*
#				Orbit-Hopper
#				Game.cpp
#
#				Copyright (C) 2004-2006 Harald Obermaier
#
# This file is part of Orbit-Hopper.
#
#   Orbit-Hopper is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   Orbit-Hopper is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Orbit-Hopper; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
*/
#ifndef WIN32
	#include "../headers/Game.h"
	#include "../headers/Gamestate.h"
	#include "../headers/Init.h"
	#include "../headers/TimeAttack.h"
	#include "../headers/LevelEd.h"
	#include "../headers/Menu.h"
	#include "../headers/Options.h"
	#include "../headers/MapSel.h"
	#include "../headers/LevelEnd.h"
	#include "../headers/Console.h"
	#include "../headers/ModeSelM.h"
	#include "../headers/CastleM.h"
	#include "../headers/CastleMEnd.h"
	#include "../headers/RaceM.h"
	#include "../headers/RaceEnd.h"
	#include "../headers/SinglePlSel.h"
	#include "../headers/Single.h"
	#include "../headers/SinglePlCSel.h"
	#include "../headers/Utils.h"
#else
	#include "..\headers\Game.h"
	#include "..\headers\Gamestate.h"
	#include "..\headers\Init.h"
	#include "..\headers\TimeAttack.h"
	#include "..\headers\LevelEd.h"
	#include "..\headers\Menu.h"
	#include "..\headers\Options.h"
	#include "..\headers\MapSel.h"
	#include "..\headers\LevelEnd.h"
	#include "..\headers\Console.h"
	#include "..\headers\ModeSelM.h"
	#include "..\headers\CastleM.h"
	#include "..\headers\CastleMEnd.h"
	#include "..\headers\RaceM.h"
	#include "..\headers\RaceEnd.h"
	#include "..\headers\SinglePlSel.h"
	#include "..\headers\Single.h"
	#include "..\headers\SinglePlCSel.h"
	#include "..\headers\Utils.h"
#endif
#include <iostream>
using namespace std;
GLuint  **anims = NULL;

Gamestate *cur_gamestate = NULL;
Gamestate *gamestates[15];

extern float *edgepts;
extern float * cornerpts;

unsigned int frames = 0;
int animcount = 0;

void SwitchGamestate(int new_state)
{
	cur_gamestate = gamestates[new_state];
	cur_gamestate->Init();
}


#ifndef WIN32
int main(int argc, char *argv[])
#else
int APIENTRY WinMain (HINSTANCE hInst, HINSTANCE hPrev, LPSTR lpCmd, int nShow)
#endif
{
	
	
	bool done = false;
	unsigned int last_time = SDL_GetTicks();
	unsigned int framecount = 0;
	unsigned int msecs = 0;
	
	
	//initialize gamestates - beware: explicit casts
	gamestates[GS_MENU] = (Gamestate *)(new Menu());
	gamestates[GS_MAPSEL] = (Gamestate *)(new MapSel());
	gamestates[GS_TIMEATTACK] = (Gamestate *)(new TimeAttack());
	gamestates[GS_SINGLE] = (Gamestate *)(new Single());
	gamestates[GS_SINGLESEL] = (Gamestate *)(new SinglePlSel());
	gamestates[GS_CONSOLE] = (Gamestate *)(new Console());
	gamestates[GS_RACEM] = (Gamestate *)(new RaceM());
	gamestates[GS_RACEEND] = (Gamestate *)(new RaceEnd());
	gamestates[GS_LEVELED] = (Gamestate *)(new LevelEd());
	gamestates[GS_LEVELEND] = (Gamestate *)(new LevelEnd());
	gamestates[GS_MODESELM] = (Gamestate *)(new ModeSelM());
	gamestates[GS_CASTLEM] = (Gamestate *)(new CastleM());
	gamestates[GS_CAEND] = (Gamestate *)(new CastleMEnd());
	gamestates[GS_OPTIONS] = (Gamestate *)(new Options());
	gamestates[GS_SINGLECSEL] = (Gamestate *)(new SinglePlCSel());
	
	gamestates[GS_CONSOLE]->Init();
	
	SwitchGamestate(GS_MENU);

	if(Init() == -1)
		return -1;
		
	SDL_EnableUNICODE(true);
	
	while(!done)
	{
		framecount++;	//count frames
		unsigned int new_time = SDL_GetTicks();
		unsigned int diff_time = new_time - last_time;
		last_time = new_time;
		
		msecs+= diff_time;
		if(msecs >= 1000) //1 sec has passed
		{
			msecs = msecs%1000;
			frames = framecount; 	//save number of frames per second
			framecount = 0;
		}
			
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);		
  		glLoadIdentity();
  		glColor3f(1.0f,1.0f,1.0f);
  		
			cur_gamestate->Render();	//do rendering 
		
		SDL_GL_SwapBuffers();
		
		
		SDL_Event event;
		while(SDL_PollEvent(&event))
		{
			if(cur_gamestate->Input(event,event.key.keysym)==-1)	//check for input
				done = true;
		}
			
		cur_gamestate->Update(diff_time);	//update
		
	}
	
	//clean up
	for(int i = 0;i<15;i++)
	{
		
		delete gamestates[i];
		gamestates[i] = NULL;
	}
		
	cur_gamestate = NULL;
	if(anims)
		delete[] anims;
	anims = NULL;
	
	delete[] cornerpts;
	delete[] edgepts;

	SDL_Quit();
	
	
	return 0;
	
}
